package de.hanser.buch.opiz.domain;

import junit.framework.TestCase;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.AnnotationConfiguration;

/**
 * Demonstriert die save-Methode.
 *
 * @author Stefan Wille
 */
public class SaveTest extends TestCase {

    public void testSave() {
        SessionFactory sessionFactory = new AnnotationConfiguration().configure().buildSessionFactory();
        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();

        Pizza pizza = new Pizza();
        pizza.setName("Thunfisch");

        session.save(pizza);

        assertNotNull(pizza.getId());

        transaction.commit();
        session.close();
        sessionFactory.close();
    }

    public void testSaveWithExceptionHandling() throws Exception {
        SessionFactory sessionFactory = new AnnotationConfiguration().configure().buildSessionFactory();
        Session session = sessionFactory.openSession();

        Transaction transaction = session.beginTransaction();
        try {
            Pizza pizza = new Pizza();
            pizza.setName("Thunfisch");
            session.save(pizza);
            assertNotNull(pizza.getId());

            transaction.commit();
        }
        catch (Exception e) {
            transaction.rollback();
            throw e;
        }
        finally {
            session.close();
        }

        sessionFactory.close();
    }

}
