package de.hanser.buch.opiz.domain;

import org.hibernate.Hibernate;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import java.io.Serializable;

/**
 * Ein Kunde.
 *
 * @author Stefan Wille
 */
@Entity
@DiscriminatorValue("Customer")
public class Customer extends BusinessPartner implements Serializable {
    private String username;

    private String password;

    private Address address;
    
    /**
     * Default-Konstruktor für Hibernate.
     */
    public Customer() {
    }

    /**
     * Demonstriert die Implementation von equals anhand des Business Keys.
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || Hibernate.getClass(this) != Hibernate.getClass(o)) return false;

        final Customer customer = (Customer) o;

        if (username != null ? !username.equals(customer.username) : customer.username != null) return false;

        return true;
    }

    /**
     * Demonstriert die Implementation von hashcode anhand des Business Keys.
     */
    @Override
    public int hashCode() {
        return (username != null ? username.hashCode() : 0);
    }

    /**
     * Der Benutzername, unter dem sich ein Kunde registriert hat.
     * <p/>
     * Ist der Business Key, deshalb optional = false.
     * <p/>
     * Demonstriert die @Column-Annotation.
     * Wegen InheritanceType.SINGLE_TABLE muss nullable = true sein.
     */
    @Basic(optional = false)
    @Column(name = "user_name", length = 63, nullable = true, unique = true)
    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    /**
     * Die Adresse des Kunden.
     */
    @OneToOne(optional = true, cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    @JoinColumn(name = "address_fk")
    public Address getAddress() {
        return address;
    }

    public void setAddress(Address address) {
        this.address = address;
    }

    /**
     * Das Passwort eines registrierten Kunden.
     */
    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }
}
