package de.hanser.buch.opiz.domain;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Domain-Objekt Pizza.
 */
@SuppressWarnings("serial")
@Entity
@Table(name = "pizza")
public class Pizza implements Serializable {

    private static final double BASE_PRICE = 3.0;

    private Integer id;

    private String name;

    /**
     * Die Bestellung, zu der diese Pizza gehrt.
     */
    private Order order;

    /**
     * Die Belge fr diese Pizza.
     */
    private List<Topping> toppings;

    /**
     * Default-Konstruktor fuer Hibernate.
     */
    public Pizza() {
        this.toppings = new ArrayList<Topping>();
    }

    public Pizza(Integer aPizzaId) {
        this();
        this.id = aPizzaId;
    }

    public Pizza(String aName, List<Topping> theToppings) {
        this.name = aName;
        this.toppings = theToppings;
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return id;
    }

    public void setId(Integer anId) {
        this.id = anId;
    }

    /**
     * Der Name der Pizza.
     */
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @ManyToOne
    @JoinColumn(name = "order_fk")
    public Order getOrder() {
        return order;
    }

    public void setOrder(Order order) {
        this.order = order;
    }

    @ManyToMany
    @JoinTable(name = "pizza_topping", joinColumns = @JoinColumn(name = "pizza_id"), inverseJoinColumns = @JoinColumn(name = "topping_id"))
    public List<Topping> getToppings() {
        return toppings;
    }

    public void setToppings(List<Topping> toppings) {
        this.toppings = toppings;
    }

    public void addTopping(Topping topping) {
        this.toppings.add(topping);
    }

    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("[").append(super.toString());
        sb.append(", id: ").append(id);
        sb.append(", name: ").append(name);
        sb.append(", toppings: [");
        for (Topping top : toppings) {
            sb.append(top).append(",");
        }
        if (order != null) {
            sb.append(", order: ").append(order.getId()).append(", ")
                    .append(order.getState());
        } else {
            sb.append(", order: null");
        }
        sb.append("]]");

        return sb.toString();
    }

    /**
     * Liefert den Preis einer Pizza. Da der Preis in Abhngigkeit der
     * Pizza-Eigenschaften berechnet wird, wird er nicht in der Datenbank
     * gespeichert.
     * 
     * Demonstriert&nbsp;@Transient.
     */
    @Transient
    public double getPrice() {
        double price = BASE_PRICE;
        for (Topping topping : toppings) {
            price += topping.getPrice();
        }
        return price;
    }

}
