package de.hanser.buch.opiz.domain;

import javax.persistence.Basic;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.io.Serializable;

/**
 * Reprsentiert einen Pizza-Belag.
 */
@SuppressWarnings("serial")
@Entity
@Table(name = "topping")
public class Topping implements Serializable {

    private Integer id;

    private String name;

    private ToppingPriceCategory priceCategory;

    /**
     * Default-Konstructor fuer Hibernate
     */
    public Topping() {
    }

    public Topping(String aName, ToppingPriceCategory aPrice) {
        this.name = aName;
        this.priceCategory = aPrice;
    }

    public Topping(Integer aToppingId) {
        this.id = aToppingId;
    }

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    /**
     * Preiskategorie fuer diesen Belag.
     */
    @Basic(optional = false)
    public ToppingPriceCategory getPriceCategory() {
        return priceCategory;
    }

    public void setPriceCategory(ToppingPriceCategory aPriceCategory) {
        this.priceCategory = aPriceCategory;
    }

    @Transient
    public double getPrice() {
        return this.priceCategory.getPrice();
    }

    @Override
    public String toString() {
        return name + "(" + priceCategory + ")";
    }
}
