package de.hanser.buch.opiz.domain;

import org.hibernate.Transaction;
import org.hibernate.Query;

import java.util.List;

/**
 * Demonstriert die bidirektionale 1:N-Beziehung zwischen Order und Pizza
 *
 * @author Stefan Wille
 */
public class BidirectionalOneToManyTest extends AbstractHibernateTest {

    public void testAssociation() {
        Transaction transaction = session.beginTransaction();

        // Erzeuge eine Bestellung
        Order order = new Order();
        session.save(order);

        // Erzeuge eine Pizza
        Pizza pizza = new Pizza();
        session.save(pizza);

        // Und stelle die Beziehung zwischen den beiden Objekten her.
        order.getPizzas().add(pizza);
        pizza.setOrder(order);

        // Fhre eine Query auf der Basis der Beziehung durch.
        Query query = session.createQuery("select p from Order o join o.pizzas p where o.id = ?");
        query.setParameter(0, order.getId());
        Pizza p = (Pizza) query.uniqueResult();
        assertSame(pizza, p);

        transaction.commit();
    }

    public void testOuterJoin() {
        Transaction transaction = session.beginTransaction();

        // Zeige die Outer-Join-Syntax
        Query query = session.createQuery("from Order o left outer join o.pizzas p");
        List list = query.list();

        transaction.commit();
    }

    public void testElements() {
        Transaction transaction = session.beginTransaction();

        // Erzeuge eine Pizza
        Pizza pizza = new Pizza();
        session.save(pizza);

        // Zeige die elements()-Syntax
        Query query = session.createQuery("select o from Order o where ? in elements(o.pizzas)");
        query.setParameter(0, pizza);
        List list = query.list();

        transaction.commit();
    }

}
