package de.hanser.buch.opiz.domain;

import org.hibernate.Transaction;
import org.hibernate.Query;

import java.util.List;

/**
 * Demonstriert die unidirektionale M:N-Beziehung zwischen Pizza und Topping.
 *
 * @author Stefan Wille
 */
public class UniDirectionalManyToManyTest extends AbstractHibernateTest {

    @SuppressWarnings({"unchecked"})
    public void testAssociation() {
        Transaction transaction = session.beginTransaction();

        // Erzeuge ein Topping
        Topping topping = new Topping();
        topping.setName("Salami");
        topping.setPriceCategory(ToppingPriceCategory.ONE);
        session.save(topping);

        // Erzeuge eine Pizza
        Pizza pizza = new Pizza();
        session.save(pizza);

        // Und stelle die Beziehung zwischen den beiden Objekten her.
        pizza.getToppings().add(topping);

        // Fhre eine Query auf der Basis der Beziehung durch.
        Query query = session.createQuery("select p from Pizza p join p.toppings t where t.name is 'Salami'");
        List<Pizza> pizzas =  query.list();
        assertEquals(1, pizzas.size());
        assertSame(pizza, pizzas.get(0));

        transaction.commit();
    }

}
