package de.hanser.buch.opiz.auth;

import org.acegisecurity.Authentication;
import org.acegisecurity.AuthenticationManager;
import org.acegisecurity.GrantedAuthority;
import org.acegisecurity.GrantedAuthorityImpl;
import org.acegisecurity.context.SecurityContext;
import org.acegisecurity.context.SecurityContextHolder;
import org.acegisecurity.context.SecurityContextImpl;
import org.acegisecurity.providers.UsernamePasswordAuthenticationToken;
import org.acegisecurity.userdetails.UserDetails;
import org.acegisecurity.userdetails.UsernameNotFoundException;
import org.springframework.dao.DataAccessException;

import de.hanser.buch.opiz.dao.CustomerDao;
import de.hanser.buch.opiz.domain.Customer;

/**
 * Service fr die Authentifizierung eines Kunden.
 */
public class CustomerAuthServiceImpl implements CustomerAuthService {

    private static GrantedAuthority[] userGrants = { new GrantedAuthorityImpl(
            CustomerAuthService.ROLE_CUSTOMER) };

    private CustomerDao customerDao;

    private AuthenticationManager authenticationManager;

    public UserDetails loadUserByUsername(String name) throws UsernameNotFoundException,
            DataAccessException {
        Customer customer = customerDao.selectCustomerByUsername(name);
        if (customer == null) {
            throw new UsernameNotFoundException(name);
        }
        return getUserDetails(customer);
    }

    public void authenticate(UserDetails userDetails, Object credentials) {

        final UsernamePasswordAuthenticationToken request = new UsernamePasswordAuthenticationToken(
                userDetails, credentials, userDetails.getAuthorities());
        final Authentication result = authenticationManager.authenticate(request);

        // Wenn ntig einen neuen Sicherheitskontext erzeugen
        if (SecurityContextHolder.getContext() == null) {
            SecurityContextHolder.setContext(new SecurityContextImpl());
        }

        // Die erfolgreiche Authentisierung im Sicherheitskontext festhalten
        final SecurityContext sc = (SecurityContext) SecurityContextHolder.getContext();
        sc.setAuthentication(result);
        SecurityContextHolder.setContext(sc);
    }

    public Customer getPrincipal() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Customer customer = null;
        if (authentication != null) {
            // Benutzer angemeldet
            CustomerUserDetail customerUserDetail = (CustomerUserDetail) authentication
                    .getPrincipal();
            customer = customerUserDetail.getCustomer();
        }
        return customer;
    }

    public UserDetails getUserDetails(Customer customer) {
        return new CustomerUserDetail(customer, userGrants);
    }

    public void setCustomerDao(CustomerDao customerDao) {
        this.customerDao = customerDao;
    }

    public void setAuthenticationManager(AuthenticationManager authenticationManager) {
        this.authenticationManager = authenticationManager;
    }

}
