package de.hanser.buch.opiz.auth;

import java.util.Iterator;

import org.acegisecurity.Authentication;
import org.acegisecurity.ConfigAttribute;
import org.acegisecurity.ConfigAttributeDefinition;
import org.acegisecurity.vote.AccessDecisionVoter;

import de.hanser.buch.opiz.domain.Customer;

public class ZipVoter implements AccessDecisionVoter {

    private static final String ZIP_PREFIX = "ZIP_";

    public boolean supports(ConfigAttribute attribute) {
        return (attribute.getAttribute() != null) 
            && attribute.getAttribute().startsWith(ZIP_PREFIX);
    }

    /**
     * Alle Klassen werden untersttzt.
     * 
     * @param clazz das abzusichernde Objekt
     * 
     * @return immer <code>true</code>
     */
    @SuppressWarnings("unchecked")
    public boolean supports(Class clazz) {
        return true;
    }

    /* (non-Javadoc)
     * @see org.acegisecurity.vote.AccessDecisionVoter#vote(org.acegisecurity.Authentication, java.lang.Object, org.acegisecurity.ConfigAttributeDefinition)
     */
    @SuppressWarnings("unchecked")
    public int vote(Authentication authentication, Object object, ConfigAttributeDefinition config) {
        int result = ACCESS_ABSTAIN;
        Iterator iter = config.getConfigAttributes();

        while (iter.hasNext()) {
            ConfigAttribute attribute = (ConfigAttribute) iter.next();

            if (this.supports(attribute)) {
                String zipGrant = attribute.toString().substring(ZIP_PREFIX.length());
                CustomerUserDetail customerUserDetail = (CustomerUserDetail) authentication
                        .getPrincipal();
                Customer customer = customerUserDetail.getCustomer();
                Integer zip = customer != null ? customer.getAddress().getZip() : null;
                String zipCompare = zip != null ? zip.toString() : "";
                result = zipCompare.startsWith(zipGrant) ? ACCESS_GRANTED : ACCESS_DENIED;
            }
        }

        return result;
    }
}
