package de.hanser.buch.opiz.web;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * Filter, um gleichzeitige Requests eines Benutzers ber die HttpSession zu
 * serialisieren.
 */
public class SerializeRequestsFilter implements Filter {

    private static final String SYNC_OBJECT_KEY = "SYNC_OBJECT_KEY";

    /** {@inheritDoc} * */
    public void init(FilterConfig config) throws ServletException {
    }

    /** {@inheritDoc} */
    public void destroy() {
    }

    /** {@inheritDoc} */
    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain) throws IOException, ServletException {
        final HttpSession session = ((HttpServletRequest) request).getSession();
        synchronized (getSynchronizationObject(session)) {
            chain.doFilter(request, response);
        }
    }

    /**
     * Besorge ein Objekt, auf dem wir uns synchronisieren knnen.
     * 
     * @param session
     *            Http-Session
     */
    private static synchronized Object getSynchronizationObject(
            HttpSession session) {
        Object syncObj = session.getAttribute(SYNC_OBJECT_KEY);
        if (syncObj == null) {
            syncObj = new Object();
            session.setAttribute(SYNC_OBJECT_KEY, syncObj);
        }
        return syncObj;
    }
}
