package de.hanser.buch.opiz.web.jsf;

import javax.faces.model.SelectItem;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Basisimplmentierung einer JSF-Auswahlliste.
 *
 * @param <T> Objekttyp in der Auswahlliste
 */
public abstract class AbstractSelectionList<T> {

    private List<SelectItem> selection;

    private Map<String, T> lookupMap;

    /**
     * Liste von SelectItem-Objekten
     */
    public Collection<SelectItem> getSelectionList() {
        if (selection == null) {
            selection = new ArrayList<SelectItem>();
            fillSelectionList();
        }
        return selection;
    }

    /**
     * Liefert die Anzahl der Elemente in der Liste.
     * @return die Anzahl
     */
    public Integer getItemCount() {
        return getSelectionList().size();
    }
    
    /**
     * Liefert zu einem Schlssel das konkrete Objekt.
     *
     * @param key der Schlssel
     * @return das Objekt
     */
    public T getObjectByKey(String key) {
        return lookupMap.get(key);
    }

    /**
     * Die abgeleitete Klasse muss das konkrete Fllen der Liste realisieren.
     *
     * @see #addSelectionItem(String, Object)
     */
    protected abstract void fillSelectionList();

    /**
     * Wird von der abgeleiteten Klasse aufgerufen, um ein Objekt fr die Auswahlliste zu
     * registrieren. Der bergebene Schlssel wird in der jetzigen Implementation ohne bersetzung
     * auch zur Anzeige der Auswahlliste verwendet.
     *
     * @param key der Schlssel, unter dem das Objekt gefunden werden kann
     * @param obj das Objekt
     */
    protected synchronized void addSelectionItem(String key, T obj) {
        selection.add(new SelectItem(obj, key));
        registerObjectByKey(key, obj);
    }

    private void registerObjectByKey(String name, T obj) {
        if (lookupMap == null) {
            lookupMap = new HashMap<String, T>();
        }
        lookupMap.put(name, obj);
	}
}
