package de.hanser.buch.opiz.web.jsf;

import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;

import org.acegisecurity.AccessDeniedException;

import de.hanser.buch.opiz.auth.CustomerAuthService;
import de.hanser.buch.opiz.dao.OrderDao;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderState;
import de.hanser.buch.opiz.domain.Pizza;
import de.hanser.buch.opiz.service.OrderService;
import de.hanser.buch.opiz.web.RequestAndSessionKeyConstants;

/**
 * Aktionen rund um Bestellungen.
 */
public class OrderAction {

    private OrderService orderService;

    private CustomerAuthService customerAuthService;
    
    private OrderDao orderDao;
    
    /**
     * Fgt der aktuellen Bestellung eine Pizza hinzu.
     * @return Aktion
     */
    @SuppressWarnings("unchecked")
    public String add() {
        FacesContext ctx = FacesContext.getCurrentInstance();
        Map sessionMap = ctx.getExternalContext().getSessionMap();
        Order order = (Order) sessionMap.get(RequestAndSessionKeyConstants.SESSION_KEY_ORDER);
        if (order == null) {
            order = new Order();
            sessionMap.put(RequestAndSessionKeyConstants.SESSION_KEY_ORDER, order);
        }

        Pizza pizza = (Pizza) ctx.getExternalContext().getRequestMap().get(
                RequestAndSessionKeyConstants.REQUEST_KEY_PIZZA);
        if (pizza.getName() == null || pizza.getName().length() == 0) {
            pizza.setName("Kein Name");
        }
        ToppingList selectedToppings = (ToppingList) ctx.getExternalContext().getRequestMap().get(
                RequestAndSessionKeyConstants.REQUEST_KEY_TOPPINGS_SELECTION);
        if (selectedToppings != null) {
            pizza.setToppings(selectedToppings.getToppings());
        }
        // Nicht direkt die Pizza aus der Liste verwenden,
        // sondern die Pizza aus der Liste nur als Vorlage verwenden und clonen.
        Pizza pizzaClone = (Pizza) pizza.clone();
        pizzaClone.setOrder(order);
        order.addPizza(pizzaClone);
        return "add";
    }

    /**
     * Ermittelt die Wartezeit einer Lieferung anhand der offenen Bestellungen. 
     * @return die Wartezeit in Minuten
     */
    public int getQueueTime() {
        // Hier werden Bestellungen geladen und implizit die Kunden zu der Bestellung.
    	// Dies kann zu einer NonUniqueObjectExceptions fhren,
    	// wenn ein Kunde danach explizit zur Session hinzugefgt werden soll 
        return orderService.getQueueTime();
    }
    
    @SuppressWarnings("unchecked")
    /**
     * Fhrt eine Bestellung fr einen angemeldeten Kunden aus.
     * Die Bestellung wird danach in der HTTP-Session wieder auf <code>null</code> gesetzt.
     * @return Aktion
     */
    public String order() {
        FacesContext ctx = FacesContext.getCurrentInstance();
        Map sessionMap = ctx.getExternalContext().getSessionMap();

        Order order = (Order) sessionMap.get(RequestAndSessionKeyConstants.SESSION_KEY_ORDER);
        if (order == null) {
            return "home";
        }
        Customer customer = customerAuthService.getPrincipal();
        if (customer != null) {
            order.setState(OrderState.OPEN);
            orderService.createOrder(order, customer.getId());
            try {
                // siehe Kapitel 9.3.6, Nebenlaeufigkeit
                Thread.sleep(2500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            sessionMap.put(RequestAndSessionKeyConstants.SESSION_KEY_ORDER, null);
        } else {
            throw new AccessDeniedException("Kein Kunde angemeldet.");
        }
        return "deliver";
    }

    /**
     * Liefert alle Bestellungen eines Kunden.
     * @return die Liste
     */
    public List<Order> getOrderHistory() {
        Customer customer = customerAuthService.getPrincipal();
        List<Order> orders = orderDao.getOrdersForCustomer(customer);
        return orders;
    }
    
    public void setOrderService(OrderService orderService) {
        this.orderService = orderService;
    }
    
    public void setCustomerAuthService(CustomerAuthService customerAuthService) {
        this.customerAuthService = customerAuthService;
    }
    
    public void setOrderDao(OrderDao orderDao) {
        this.orderDao = orderDao;
    }
}
