package de.hanser.buch.opiz.web.jsf;

import java.util.Arrays;
import java.util.List;

import de.hanser.buch.opiz.dao.ToppingDao;
import de.hanser.buch.opiz.domain.Topping;

/**
 * Klasse zum Halten einer Liste von Belgen.
 * Die globale Instanz im Application-Scope hlt eine Liste der verfgbaren Belge.
 * Die Instanz im Request-Scope hlt eine Liste ausgewhlter Belge, wenn sich
 * ein Kunde eine Pizza zusammenstellt.
 * ber die Array-Methoden {@link #getToppingArr()} und {@link #setToppingArr(Topping[])}
 * kann MyFaces die Liste aus einer Auswahl in einer <code>h:selectManyCheckbox</code>
 * unter Benutzung des Konverters {@link ToppingConverter} direkt mit Topping-Objekten fllen.
 * Die List-Methoden {@link #getToppings()} und {@link #setToppings(List))} reichen MyFaces zur
 * korrekten Konvertierung nicht.
 *   
 * @see ToppingConverter    
 */
public class ToppingList extends AbstractSelectionList<Topping> {

	private ToppingDao toppingDao;
	
    private List<Topping> toppings;

    protected void fillSelectionList() {
    	List<Topping> result = toppingDao.selectAll();
    	for(Topping topping : result) {
            addSelectionItem(topping);
    	}
    }
    
    // Fgt das konkrete Objekt der Liste hinzu
    private void addSelectionItem(Topping topping) {
        addSelectionItem(topping.getName(), topping);
    }
    
    /**
     * Liefert eine Array-Reprsentation der Belge.
     * Dies ist notwendig, damit die Konvertierung bei der h:selectManyCheckbox
     * von ausgewhlten Belgen korrekt funktioniert.
     * @return das Array
     */
    public Topping[] getToppingArr() {
        return toppings != null ? toppings.toArray(new Topping[0]) : null;
    }

    public void setToppingArr(Topping[] toppings) {
        this.toppings = Arrays.asList(toppings);
    }

    public List<Topping> getToppings() {
        return toppings;
    }

    public void setToppings(List<Topping> toppings) {
        this.toppings = toppings;
    }

    public void setToppingDao(ToppingDao toppingDao) {
        this.toppingDao = toppingDao;
    }
}
