package de.hanser.buch.opiz.web.jsf;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.reset;
import static org.easymock.EasyMock.verify;

import javax.faces.context.FacesContext;

import org.acegisecurity.AccessDeniedException;

import de.hanser.buch.opiz.auth.CustomerAuthService;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.service.OrderService;
import de.hanser.buch.opiz.web.RequestAndSessionKeyConstants;
import de.hanser.buch.test.BaseJsfSpringTestCase;

public class OrderActionTest extends BaseJsfSpringTestCase {

    protected String[] getContextLocations() {
        return new String[] { "classpath:applicationContext.xml",
                "classpath:applicationContextJSF.xml",
                "classpath:applicationContextTest.xml" };
    }

    private OrderAction getOrderAction() {
        return (OrderAction) getAppContext("default")
                .getBean("orderAction");
    }

    @SuppressWarnings("unchecked")
    private void placeOrderInSession(Order order) {
        FacesContext.getCurrentInstance().getExternalContext().getSessionMap()
                .put(RequestAndSessionKeyConstants.SESSION_KEY_ORDER, order);
    }

    /**
     * Test auf Fehler bei unberechtigtem Zugriff.
     * 
     * @throws Exception Exception im Test
     */
    public void testOrderFailAccessDenied() throws Exception {
        placeOrderInSession(new Order());
        OrderAction orderAction = getOrderAction();
        try {
            orderAction.order();
            fail();
        } catch (AccessDeniedException e) {
        }
    }

    /**
     * Test auf Fehler, weil der Inhalt der Http-Session korrupt ist.
     * 
     * @throws Exception Exception im Test
     */
    public void testOrderFailNoOrderInSession() throws Exception {
        OrderAction orderAction = getOrderAction();
        String result = orderAction.order();
        // Bei einer fehlenden Bestellung wird auf die Hauptseite verzweigt
        assertEquals("home", result);
    }

    /**
     * Test der korrekten Erzeugung einer Bestellung.
     * 
     * @throws Exception Exception im Test
     */
    public void testOrderSuccess() throws Exception {
        CustomerAuthService customerAuthService 
            = (CustomerAuthService) getMockFromAppContext("customerAuthService");
        Customer customer = new Customer();
        Integer customerId = new Integer(4711);
        customer.setId(customerId);
        expect(customerAuthService.getPrincipal()).andReturn(customer);
        replay(customerAuthService);

        Order order = new Order();
        OrderService orderService 
            = (OrderService) getMockFromAppContext("orderService");
        orderService.createOrder(order, customerId);
        replay(orderService);

        placeOrderInSession(order);
        OrderAction orderAction = getOrderAction();
        String result = orderAction.order();
        assertEquals("deliver", result);

        verify(orderService);
    }

    private Object getMockFromAppContext(String beanName) {
        Object mock = getAppContext("default").getBean(beanName);
        reset(mock);
        return mock;
    }
}
