package de.hanser.buch.opiz.ejb;

import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.SessionContext;

import org.springframework.context.access.ContextSingletonBeanFactoryLocator;
import org.springframework.ejb.support.AbstractStatelessSessionBean;

import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderFilter;
import de.hanser.buch.opiz.service.OrderService;

/**
 * EJB fuer den OrderService.
 *
 */
public class OrderServiceEjb extends AbstractStatelessSessionBean
        implements OrderService {

	private static final long serialVersionUID = 1L;
	
	private OrderService orderService = null;

    @Override
    protected void onEjbCreate() throws CreateException {
        orderService = (OrderService) getBeanFactory().getBean(
                "orderService", OrderService.class);
    }

    public List<Order> selectOrder(OrderFilter filter) {
        return orderService.selectOrder(filter);
    }

    public Integer createPendingOrder(Integer aPizzaId) {
        return orderService.createPendingOrder(aPizzaId);
    }

    public boolean finishOrder(Integer anOrderId) {
        return orderService.finishOrder(anOrderId);
    }

    public double getPrice(Integer anOrderId) {
        return orderService.getPrice(anOrderId);
    }

    public boolean addPizzaToOrder(Integer anOrderId, Integer aPizzaId) {
        return orderService.addPizzaToOrder(anOrderId, aPizzaId);
    }

    public void createOrder(Order o) {
        // nicht in der EJB implementiert
    }

    public void createOrder(Order o, Integer custId) {
        // nicht in der EJB implementiert
    }

    public int getQueueTime() {
        // nicht in der EJB implementiert
        return 0;
    }
    
    /**
     * Default BeanFactoryLocator wird hier ueberschriben,
     * um auf ContextSingletonBeanFactory umzusteigen.
     * <p>
     * TODO: Eine XML-Datei beanRefContext.xml muss im Klassenpfad liegen,
     * dessen Inhalt unsere applicationContext referenziert wie folgt:
     <pre>
       &lt;beans&gt;
       &nbsp;&nbsp;&lt;bean id="opizBeanFactory" class="org.springframework.context.support.ClassPathXmlApplicationContext"&gt;
       &nbsp;&nbsp;&nbsp;&nbsp;&lt;constructor-arg value="applicationContext.xml" /&gt;
       &nbsp;&nbsp;&lt;/bean&gt;
       &lt;/beans&gt;
     </pre>
     * @see javax.ejb.SessionBean#setSessionContext(javax.ejb.SessionContext)
     */
//    @Override
//    public void setSessionContext(SessionContext sessionContext) {
//        super.setSessionContext(sessionContext);
//        setBeanFactoryLocator(ContextSingletonBeanFactoryLocator.getInstance());
//        setBeanFactoryLocatorKey("opizBeanFactory");
//    }

}
