package de.hanser.buch.opiz.ejb;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.ejb.EJBException;

import org.apache.log4j.Logger;
import org.springframework.beans.BeansException;
import org.springframework.ejb.support.AbstractStatefulSessionBean;

import de.hanser.buch.opiz.service.VisitorService;

public class VisitorServiceEjb extends AbstractStatefulSessionBean
        implements VisitorService {

	private static final long serialVersionUID = 1L;

	private static final Logger LOG = Logger
            .getLogger(VisitorServiceEjb.class);

    private VisitorService visitorService = null;

    /**
     * delegiert an die visitorService-Bean
     */
    public void addVisitor() {
        visitorService.addVisitor();
    }

    /**
     * delegiert an die visitorService-Bean
     */
    public void removeVisitor() {
        visitorService.removeVisitor();
    }

    /**
     * delegiert an die visitorService-Bean
     */
    public int getNbrVisitors() {
        return visitorService.getNbrVisitors();
    }

    public void ejbCreate() throws CreateException {
        try {
            create();
        } catch (BeansException x) {
            throw new CreateException(x.getMessage());
        }
    }

    public void ejbActivate() throws EJBException, RemoteException {
        LOG.info("ejbActivate called");
        create();
    }

    private void create() {
        LOG.info("create called");
        if (visitorService == null) {
            LOG.info("loading bean factory");
            loadBeanFactory();
            visitorService = (VisitorService) getBeanFactory()
                    .getBean("visitorService");
        } else
            LOG.info("NOT re-loading bean factory");

    }

    public void ejbPassivate() throws EJBException, RemoteException {
        LOG.info("ejbPassivate called");
        unloadBeanFactory();
        setBeanFactoryLocator(null);
    }

}
