package de.hanser.buch.opiz.rmi;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import de.hanser.buch.opiz.dao.CustomerDao;
import de.hanser.buch.opiz.dao.OrderDao;
import de.hanser.buch.opiz.dao.PizzaDao;
import de.hanser.buch.opiz.dao.ToppingDao;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.Pizza;
import de.hanser.buch.opiz.domain.Topping;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;

/**
 * Laedt den Application-Context und wartet.
 * 
 * Der RMI-Server in App-Ctx wird von Spring gestartet und kann dann
 * Client-Anfragen bearbeiten.
 * <p>
 * NB: da diese Projekt die opiz-Projekt referenziert, sind die log4j.xml und
 * applicationContext.xml aus opiz automatisch auf dem Klassenpfad.
 * 
 * @author dr richard oates
 * 
 */
public class SimpleServer {

    private PizzaDao pizzaDao;
    private OrderDao orderDao;
    private CustomerDao customerDao;
    private ToppingDao toppingDao;

    public void run() {

        setupDatabase();
        System.out.println("SERVER WARTET ...");

        while (true) {
            try {
                Thread.sleep(10000);
            } catch (InterruptedException x) {
                // ignore
            }
        }
    }

    private void setupDatabase() {
        Topping aTopping = new Topping("Salami-Topping", ToppingPriceCategory.ONE);
        toppingDao.saveTopping(aTopping);
        
        Pizza aPizza = new Pizza();
        aPizza.setName("Salami");
        aPizza.addTopping(aTopping);
        pizzaDao.savePizza(aPizza);
        
        Customer customer = new Customer();
        customer.setName("oates");
        customerDao.saveCustomer(customer);
        
        Order order = new Order();
        order.setCustomer(customer);
        orderDao.saveOrder(order);
    }
 
    public void setOrderDao(OrderDao anOrderDao) {
        this.orderDao = anOrderDao;
    }
 
    public void setPizzaDao(PizzaDao aPizzaDao) {
        this.pizzaDao = aPizzaDao;
    }
    
    public void setToppingDao(ToppingDao aToppingDao) {
        this.toppingDao = aToppingDao;
    }

    public void setCustomerDao(CustomerDao aCustomerDao) {
        this.customerDao = aCustomerDao;
    }

    public static void main(String[] args) {

        // lade den normalen App-Ctx (aus opiz) und die RMI-Ergaenzung dazu
        ApplicationContext appCtx = new ClassPathXmlApplicationContext(
                new String[] { "applicationContext.xml",
                        "context-rmi.xml" });

        SimpleServer server = (SimpleServer) appCtx.getBean("server");
        server.run();
    }
}
