package de.hanser.buch.opiz.web;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import de.hanser.buch.opiz.dao.CustomerDao;
import de.hanser.buch.opiz.dao.PizzaDao;
import de.hanser.buch.opiz.dao.ToppingDao;
import de.hanser.buch.opiz.domain.Address;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.Pizza;
import de.hanser.buch.opiz.domain.Topping;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;
import de.hanser.buch.opiz.service.OrderService;

/**
 * Initialisierungs-Servlet (wird beim Start der Anwendung aufgerufen).
 */
@SuppressWarnings("serial")
public class ApplicationInit extends HttpServlet {

    /**
     * Wird beim Start des Servlets aufgerufen.
     * 
     * @throws ServletException
     *             Fehler bei der Initialisierung
     */
    public void init() throws ServletException {
        WebApplicationContext appContext = WebApplicationContextUtils
                .getWebApplicationContext(getServletContext());
        Topping jalapenho = new Topping("Jalapeno", ToppingPriceCategory.THREE);
        ToppingDao toppingDao = (ToppingDao) appContext.getBean("toppingDao");
        toppingDao.saveTopping(jalapenho);

        for (int i = 0; i < 3; i++) {
            createSampleOrder(appContext, i, jalapenho);
        }
    }

    private void createSampleOrder(WebApplicationContext appContext, int index,
            Topping jalapenho) {
        Address address = new Address("Strae " + index, 20000 + index, "Hamburg");
        Customer customer = new Customer("Horst Hungrig" + index, "hh" + index, "HHHigP.",
                address);
        CustomerDao customerDao = (CustomerDao) appContext
                .getBean("customerDao");
        customerDao.saveCustomer(customer);

        List<Topping> diaboloToppings = new ArrayList<Topping>();
        diaboloToppings.add(jalapenho);
        Pizza diabolo = new Pizza("Nummer " + index, diaboloToppings);
        PizzaDao pizzaDao = (PizzaDao) appContext.getBean("pizzaDao");
        pizzaDao.savePizza(diabolo);

        OrderService orderService = (OrderService) appContext
                .getBean("orderService");
        Order order = new Order();
        orderService.createOrder(order, customer.getId());
        orderService.addPizzaToOrder(order.getId(), diabolo.getId());
        orderService.finishOrder(order.getId());
    }
}
