package de.hanser.buch.opiz.web;

import java.io.Serializable;
import java.util.List;

import javax.faces.model.DataModel;
import javax.faces.model.DataModelEvent;
import javax.faces.model.DataModelListener;

/**
 * Diese Datei ist aus dem Beipsiel zu Spring Webflow kopiert.
 */
public class SerializableListDataModel<T> extends DataModel implements
        Serializable {

    private static final long serialVersionUID = 1L;

    private int currentIndex = -1;

    private int lastSelectedIndex = -1;

    private List<T> _data;

    public SerializableListDataModel() {
        super();
    }

    /**
     * Adapt the list to a data model;
     * 
     * @param list
     *            the list
     */
    public SerializableListDataModel(List<T> list) {
        if (list == null)
            throw new NullPointerException("list");
        setWrappedData(list);
    }

    public int getRowCount() {
        if (_data == null) {
            return -1;
        }
        return _data.size();
    }

    public Object getRowData() {
        return getData(currentIndex);
    }

    public T getLastSelectedData() {
        return getData(lastSelectedIndex);
    }

    public int getRowIndex() {
        return currentIndex;
    }

    public Object getWrappedData() {
        return _data;
    }

    public boolean isRowAvailable() {
        return isRowAvailable(currentIndex);
    }

    private boolean isRowAvailable(int index) {
        if (_data == null) {
            return false;
        }
        return index >= 0 && index < _data.size();
    }

    public void setRowIndex(int rowIndex) {
        if (rowIndex < -1) {
            throw new IllegalArgumentException("illegal rowIndex " + rowIndex);
        }
        int oldRowIndex = currentIndex;
        currentIndex = rowIndex;
        if (currentIndex == -1 && oldRowIndex != -1) {
            lastSelectedIndex = oldRowIndex;
        }

        DataModelListener[] listeners = getDataModelListeners();
        if (_data != null && oldRowIndex != currentIndex && listeners != null) {
            Object data = isRowAvailable() ? getRowData() : null;
            DataModelEvent event = new DataModelEvent(this, currentIndex, data);
            for (int i = 0; i < listeners.length; i++) {
                listeners[i].rowSelected(event);
            }
        }
    }

    @SuppressWarnings("unchecked")
    public void setWrappedData(Object data) {
        _data = (List<T>) data;
        int rowIndex = _data != null ? 0 : -1;
        setRowIndex(rowIndex);
    }

    private T getData(int index) {
        if (_data == null) {
            return null;
        }
        if (!isRowAvailable(index)) {
            throw new IllegalArgumentException("row is unavailable");
        }
        return _data.get(index);
    }
    
}
