package de.hanser.buch.opiz.web.swf;

import java.util.List;

import org.springframework.webflow.action.MultiAction;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

import de.hanser.buch.opiz.dao.OrderDao;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderState;
import de.hanser.buch.opiz.web.SerializableListDataModel;

/**
 * Aktionen rund um Bestellungen.
 */
public class AdminOrderAction extends MultiAction {

    private OrderDao orderDao;
    
    public void setOrderDao(OrderDao orderDao) {
        this.orderDao = orderDao;
    }
    
    public Event findOpenOrders(RequestContext context) {
        SerializableListDataModel<Order> model = getOpenOrdersInFlow(context);
        // Haben wir schon unsere offenen Bestellungen im Flow?
        if (model == null)  {
            List<Order> orders = orderDao.getOrdersInState(OrderState.OPEN);
            model = new SerializableListDataModel<Order>();
            model.setWrappedData(orders);
            context.getFlowScope().put("openOrders", model);
        }
        return success();
    }

    public Event selectOpenOrder(RequestContext context) {
        SerializableListDataModel<Order> model = getOpenOrdersInFlow(context);
        Order currentOrder = model.getLastSelectedData();
        context.getFlowScope().put("currentOrder", currentOrder);
        return success();
    }

    @SuppressWarnings("unchecked")
    private SerializableListDataModel<Order> getOpenOrdersInFlow(
            RequestContext context) {
        SerializableListDataModel<Order> model = (SerializableListDataModel<Order>) context
                .getFlowScope().get("openOrders");
        return model;
    }

    @SuppressWarnings("unchecked")
    public Event deliverOrder(RequestContext context) {
        Order currentOrder = (Order) context
                .getFlowScope().get("currentOrder");
        currentOrder.deliverOrder();
        return success();
    }
    
}
