package de.hanser.buch.opiz.dao;

import static de.hanser.buch.opiz.Constants.FUNC_LOG;
import static de.hanser.buch.opiz.Constants.LOG_MSG_CUST_CHANGED;
import static de.hanser.buch.opiz.Constants.LOG_MSG_CUST_NEW;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.Session;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

import java.sql.SQLException;
import java.util.List;

/**
 * Implementation von CustomerDao.
 */
public class HibernateCustomerDaoImpl extends HibernateDaoSupport
        implements CustomerDao {
	
	private static final Logger LOG = Logger.getLogger(HibernateCustomerDaoImpl.class);
	
	
    public Customer getCustomer(Integer customerId) {
        return (Customer) getHibernateTemplate().get(Customer.class,
                customerId);
    }

    public Integer saveCustomer(Customer customer) {
        Integer ret = (Integer) getHibernateTemplate().save(customer);
        FUNC_LOG.info(LOG_MSG_CUST_NEW + ret);
        return ret;
    }

    public void updateCustomer(Customer customer) {
        getHibernateTemplate().update(customer);
        FUNC_LOG.info(LOG_MSG_CUST_CHANGED + customer.getId());
    }

    @SuppressWarnings("unchecked")
    public Customer selectCustomerByUsername(String name) {
        List<Customer> list = getHibernateTemplate().find(
                "from Customer where username = ?", name);
        if (list.size() == 0) {
            return null;
        } else if (list.size() == 1) {
            Customer customer = list.get(0);
            // Hibernate.initialize(customer.getAddress());
            return customer;
        } else {
        	LOG.debug("Doppelter Benutzername");
            throw new DataIntegrityViolationException(
                    "Mehr als ein Kunde mit demselben Benutzernamen.");
        }
    }

    @SuppressWarnings("unchecked")
    public List<Customer> getAllCustomers() {
        return getHibernateTemplate().loadAll(Customer.class);
    }

    public void attachCustomer(Customer customer) {
        getHibernateTemplate().lock(customer, LockMode.NONE);
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public int getNumberOrders(final Customer customer) {
        List result = (List) getHibernateTemplate().execute(
                new HibernateCallback() {

                    public Object doInHibernate(Session session)
                            throws HibernateException, SQLException {
                        Criteria criteria = session
                                .createCriteria(Order.class);
                        criteria.setProjection(Projections
                                .projectionList().add(
                                        Projections.rowCount()));
                        criteria.createCriteria("customer").add(
                                Restrictions.eq("id", customer
                                        .getId()));

                        return criteria.list();
                    }

                });
        return (Integer) result.get(0);
    }
}
