package de.hanser.buch.opiz.dao;

import java.sql.SQLException;
import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

import de.hanser.buch.opiz.domain.Topping;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;

/**
 * Implementierung der Schnittstelle HibernateToppingDAO.
 * <p>
 * 
 * @author dr richard oates
 */
public class HibernateToppingDaoImpl extends HibernateDaoSupport
        implements ToppingDao {

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public List<Topping> getToppingByName(String name) {
        return (List<Topping>) getHibernateTemplate()
                .findByNamedParam(
                        "from Topping where name like :name", "name",
                        "%" + name + "%");
    }

    /**
     * {@inheritDoc}
     */
    public Topping getTopping(Integer id) {
        return (Topping) getHibernateTemplate()
                .get(Topping.class, id);
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public List<Topping> selectAll() {
        return getHibernateTemplate().loadAll(Topping.class);
    }

    /**
     * {@inheritDoc}
     */
    public int saveTopping(Topping aTopping) {
        return (Integer) getHibernateTemplate().save(aTopping);
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public List<Topping> selectByNameAndPrice(String name,
            ToppingPriceCategory category) {
        List<Topping> list;

        // nicht gut:
        // list = getHibernateTemplate().find(
        // "from Topping where name like '" + name + "' and priceCategory=" +
        // category.toString());

        // unnamed parameters:
        // list = getHibernateTemplate().find(
        // "from Topping where name like ? and priceCategory = ?",
        // new Object[] {name, category});

        // named parameters:
        // list = getHibernateTemplate().findByNamedParam(
        // "from Topping where name like :name and priceCategory = :category",
        // new String[] { "name", "category" }, new Object[] { name, category
        // });

        // named query parameters:
        list = getHibernateTemplate().findByNamedQueryAndNamedParam(
                "byNameAndPrice",
                new String[] { "name", "category" },
                new Object[] { name, category });

        return list;
    }

    @SuppressWarnings("unchecked")
    public List<Topping> selectByNameAndPricePaged(final String name,
            final ToppingPriceCategory category, final int startPage,
            final int pageSize) {
        List<Topping> list;

        list = getHibernateTemplate().executeFind(
                new HibernateCallback() {

                    public Object doInHibernate(Session session)
                            throws HibernateException, SQLException {
                        Query query = session
                                .getNamedQuery("byNameAndPrice");
                        query.setString("name", name);
                        query.setInteger("category", category
                                .ordinal());

                        query.setFirstResult(startPage * pageSize);
                        query.setMaxResults(pageSize);
                        return query.list();
                    }

                });

        return list;
    }

    @SuppressWarnings("unchecked")
    public List getNumberToppingsOrderedByPrice() {
        return getHibernateTemplate().find("select new Map(top.priceCategory as category, count(*) as anzahl) " +
                                        " from Topping top group by top.priceCategory " +
                                        " order by top.priceCategory");
    }
}
