package de.hanser.buch.opiz.dao;

import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderFilter;
import de.hanser.buch.opiz.domain.OrderState;
import org.aspectj.lang.annotation.Pointcut;

import java.util.List;

/**
 * Die Order-DAO-Schnittstelle.
 * <p>
 * @author dr richard oates
 */
public interface OrderDao {

    Order getOrder(Integer orderId);

    /**
     * Speichert eine Order in der DB.
     * @param anOrder - das Order Objekt zu speichern.
     * @return der Primaerschluessel des gespeicherten Objekts.
     */
    @Pointcut("execution(* de.hanser.buch.opiz.dao.*Order*.saveOrder(de.hanser.buch.opiz.domain.Order))")
    Integer saveOrder(Order anOrder);

    List<Order> getOrdersInState(final OrderState state);

    List<Order> getOrdersForCustomer(final Customer customer);

    /**
     * Gibt eine Liste von Order-Objekten zurueck, die das Filter entsprechen.
     * Beispielmethode um Benutzung der Criteria-API zu zeigen.
     * 
     * @param filter
     *            das Filter-Objekt.
     * @return eine Liste von Order-Objekten.
     */
    List<Order> selectOrder(OrderFilter filter);

    /**
     * Gibt eine Liste von Order-Objekten zurueck,
     * die das Filter entsprechen.
     * <p/>
     * Andere Variante von selectOrder, die ein Order-Objekt als Example nimmt.
     *
     * @param exampleOrder ein Order-Objekt als Beispiel.
     * @return eine Liste von Order-Objekten.
     */
    List<Order> selectOrderByExample(Order exampleOrder);


}
