package de.hanser.buch.opiz.dao;

import java.util.List;

import de.hanser.buch.opiz.domain.Topping;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;

/**
 * Die Schnittstelle für die Topping-DAO.
 * @author dr richard oates
 *
 */
public interface ToppingDao {

    /**
     * Liest alle Belaege mit angegebenen (Teil-)Name aus der Datenbank.
     * 
     * @param name
     *            der Name des gewuenschten Belags.
     * @return eine Liste Belaege.
     */
    List<Topping> getToppingByName(String name);

    /**
     * Liest einen Belag aus der Datenbank.
     * 
     * @param id
     *            der PK des gewuenschten Belags.
     * @return der Belag.
     */
    Topping getTopping(Integer id);

    /**
     * Liefert eine Liste aller Belaege.
     * 
     * @return eine Liste aller Belaege.
     */
    List<Topping> selectAll();

    /**
     * Speichert einen Belag.
     * 
     * @param aTopping
     *            der Belag
     * @return die eindeutige Id
     */
    int saveTopping(Topping aTopping);

    /**
     * Gibt zurueck alle Belaege in der DB, die
     * die angegebenen Name und Preis-Kategorie haben.
     * @param name
     * @param category
     * @return
     */
    List<Topping> selectByNameAndPrice(String name, ToppingPriceCategory category);
    
    /**
     * Wie selectByNameAndPrice, mit Paging.
     * @param name
     * @param category
     * @param startPage die Anfangsseite
     * @param pageSize die Seitengrösse
     * @return
     */
    List<Topping> selectByNameAndPricePaged(final String name,
            final ToppingPriceCategory category, final int startPage, final int pageSize);
    
    @SuppressWarnings("unchecked")
    List getNumberToppingsOrderedByPrice();
}
