package de.hanser.buch.opiz.domain;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.Version;

/**
 * Ein Geschaeftspartner - Basisklasse fuer Customer und Supplier.
 *  
 * @author Stefan Wille
 */
@Entity
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
@DiscriminatorColumn(name = "discriminator")
public class BusinessPartner implements Serializable {

    private static final long serialVersionUID = 1L;

    private Integer id;

    private String name;

    private Integer version;
    
    /**
     * Default-Konstruktor fuer Hibernate.
     */
    public BusinessPartner() {
    }

    
    /**
     * Konstruktor zur Initialisierung.
     * @param name der Name
     */
    public BusinessPartner(String name) {
        super();
        this.name = name;
    }

    /**
     * Der Primaerschluessel des Kunden.
     */
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    /**
     * Der Name des BusinessPartner.
     */
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Version
    @Column(name="OPTLOCK")
    public Integer getVersion() {
        return version;
    }    
    
    public void setVersion(Integer version) {
        this.version = version;
    }
}
