package de.hanser.buch.opiz.service;

import de.hanser.buch.opiz.dao.CustomerDao;
import de.hanser.buch.opiz.dao.OrderDao;
import de.hanser.buch.opiz.dao.PizzaDao;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderFilter;
import de.hanser.buch.opiz.domain.OrderState;
import de.hanser.buch.opiz.domain.Pizza;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


/**
 * Service rund um Bestellungen.
 */
public class OrderServiceImpl implements OrderService {

	private PizzaDao pizzaDao;
	private OrderDao orderDao;
	private CustomerDao customerDao;
		
    /**
     * {@inheritDoc}
     */
	public void createOrder(Order o, Integer custId) {
		Customer c = customerDao.getCustomer(custId);
		o.setCustomer(c);
        for(Pizza p : o.getPizzas()) {
            pizzaDao.savePizza(p);
        }
		orderDao.saveOrder(o);
	}
	
	/**
     * {@inheritDoc}
     */
    public Integer createPendingOrder(Integer pizzaId) {
        Pizza pizza = pizzaDao.getPizza(pizzaId);
        Order order = new Order();
        order.addPizza(pizza);
        Integer newOrderId = orderDao.saveOrder(order);
        pizza.setOrder(order);
        
        return newOrderId;
    }

    /**
     * {@inheritDoc}
     */
    public boolean addPizzaToOrder(Integer anOrderId, Integer aPizzaId) {
        Pizza pizza = pizzaDao.getPizza(aPizzaId);
        if (pizza == null) {
            throw new UnknownPizzaException("Pizza ID " + aPizzaId + " nicht bekannt");            
        }
        Order order = orderDao.getOrder(anOrderId);
        if (order == null) {
            throw new UnknownOrderException("Order ID " + anOrderId + " nicht bekannt");
        }
        return order.addPizza(pizza);
    }
    
    /**
     * {@inheritDoc}
     */
    public double getPrice(Integer anOrderId) {
        Order order = orderDao.getOrder(anOrderId);
        return order.getPrice();
    }

    /**
     * {@inheritDoc}
     */
    public boolean finishOrder(Integer anOrderId) {
        Order order = orderDao.getOrder(anOrderId);
        return order.finishOrder();
    }
    
    /**
     * {@inheritDoc}
     */
    public List<Order> selectOrder(OrderFilter filter) {
        List<Order> list = orderDao.selectOrder(filter);
        // stelle sicher, dass moegliche Lazy-Objekten geladen werden
        for (Order order : list) {
            order.getPizzas();
        }
        return list;
    }
    
    @Transactional(propagation=Propagation.REQUIRED, readOnly=true)
    public int getQueueTime() {
        List<Order> orders = orderDao.getOrdersInState(OrderState.OPEN);
        return 10 + orders.size() * 2;
    }

	public void setCustomerDao(CustomerDao customerDao) {
		this.customerDao = customerDao;
	}

	public void setOrderDao(OrderDao orderDao) {
		this.orderDao = orderDao;
	}

	public void setPizzaDao(PizzaDao pizzaDao) {
		this.pizzaDao = pizzaDao;
	}
	
}
