package de.hanser.buch.opiz.dao;

import org.springframework.test.annotation.AbstractAnnotationAwareTransactionalTests;

import de.hanser.buch.opiz.domain.Address;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.OrderState;

/**
 * Test fuer HibernateCustomerDaoImpl.
 */
public class HibernateCustomerDaoImplTest extends
		AbstractAnnotationAwareTransactionalTests {
	private CustomerDao customerDao;

	@Override
	protected void onSetUpInTransaction() throws Exception {
	}

	public void testSaveCustomer() {
		Customer customer = new Customer();
		customer.setUsername("Winfried_Winnefree");
		customerDao.saveCustomer(customer);
	}

	public void testSaveCustomerWithAddress() {
		Customer customer = new Customer();
		customer.setUsername("Winfried_Winnefree");
		Address address = new Address();
		address.setStreet("Gartenweg 1");
		address.setZip(null);
		address.setCity("Bokelsen");
		customer.setAddress(address);
		customerDao.saveCustomer(customer);
	}

	public void testLoadCustomer() {
		JDBCTestHelper.insertCustomer(jdbcTemplate, 1, "Joe", "joe");

		Customer loadedCustomer = customerDao.getCustomer(1);
		assertEquals(new Integer(1), loadedCustomer.getId());
		assertEquals("Joe", loadedCustomer.getName());
	}

	/** test aggregat Funktion */
	public void testGetNumberOrders() {
		JDBCTestHelper.insertCustomer(jdbcTemplate, 1, "Joe", "joe");
		JDBCTestHelper.insertOrder(jdbcTemplate, 1, OrderState.OPEN, 1);
		JDBCTestHelper.insertOrder(jdbcTemplate, 2, OrderState.OPEN, 1);

		Customer customer = customerDao.getCustomer(1);
		assertEquals("falscher Anzahl Bestellungen", 2, customerDao
				.getNumberOrders(customer));
	}

	public void setCustomerDao(CustomerDao customerDao) {
		this.customerDao = customerDao;
	}

	@Override
	protected String[] getConfigLocations() {
		return new String[] { "applicationContext.xml" };
	}

}
