package de.hanser.buch.opiz.dao;

import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.test.annotation.AbstractAnnotationAwareTransactionalTests;

import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderFilter;
import de.hanser.buch.opiz.domain.OrderState;
import de.hanser.buch.opiz.domain.Pizza;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;

public class HibernateOrderDaoImplTest extends
		AbstractAnnotationAwareTransactionalTests {
	private static final Logger LOG = Logger
			.getLogger(HibernateOrderDaoImplTest.class);

	private OrderDao orderDao;

	private CustomerDao customerDao;

	@Override
	protected void onSetUpInTransaction() throws Exception {
		JDBCTestHelper.insertCustomer(jdbcTemplate, 1, "Katrin Kufe", "kkufe");
		JDBCTestHelper.insertCustomer(jdbcTemplate, 2, "F. Oates", "foates");

		JDBCTestHelper.insertTopping(jdbcTemplate, 1, "Tomato sauce",
				ToppingPriceCategory.ONE);
		JDBCTestHelper.insertTopping(jdbcTemplate, 2, "Edam",
				ToppingPriceCategory.TWO);
		JDBCTestHelper.insertTopping(jdbcTemplate, 3, "Salami",
				ToppingPriceCategory.TWO);
		JDBCTestHelper.insertTopping(jdbcTemplate, 4, "Onion",
				ToppingPriceCategory.ONE);
		JDBCTestHelper.insertTopping(jdbcTemplate, 5, "Tuna",
				ToppingPriceCategory.THREE);
		JDBCTestHelper.insertTopping(jdbcTemplate, 6, "Mushrooms",
				ToppingPriceCategory.TWO);

		// erste Bestellung:
		JDBCTestHelper.insertOrder(jdbcTemplate, 1, OrderState.OPEN, 1);
		JDBCTestHelper.insertPizza(jdbcTemplate, 1, "Margherita",
				new int[] { 1 }, 1);

		JDBCTestHelper.insertPizza(jdbcTemplate, 2, "Salami", new int[] { 1, 3,
				4 }, 1);

		// zweite Bestellung
		JDBCTestHelper.insertOrder(jdbcTemplate, 2, OrderState.OPEN, 2);
		JDBCTestHelper.insertPizza(jdbcTemplate, 3, "Margherita",
				new int[] { 1 }, 2);

	}

	public void testSaveOrder() {
		Order order = new Order();
		Integer res = orderDao.saveOrder(order);
		assertTrue("Falscher Ergebnis vom DAO (" + res + ")", res != 0);
	}

	public void testSimpleCreate() {
		Order order = new Order();
		Pizza pizza = new Pizza();
		order.getPizzas().add(pizza);
		pizza.setOrder(order);
		orderDao.saveOrder(order);
	}

	public void testSaveOrderWithCustomer() {
		Customer customer = new Customer();
		customer.setName("Angelo M�rkle");

		Order order1 = new Order();
		Pizza aPizza1 = new Pizza(1);
		order1.addPizza(aPizza1);
		order1.setCustomer(customer);

		Order order2 = new Order();
		Pizza aPizza2 = new Pizza(2);
		order2.addPizza(aPizza2);
		order2.setCustomer(customer);

		Integer res1 = orderDao.saveOrder(order1);
		Integer res2 = orderDao.saveOrder(order2);

		LOG.debug("saved 1st order: " + res1 + ", 2nd: " + res2);
	}

	public void testGetOrdersInState() {
		LOG.debug("testGetOrdersInState");
		List<Order> list = orderDao.getOrdersInState(OrderState.OPEN);
		assertEquals("falsche Anzahl Elemente in List: ", 2, list.size());
	}

	public void testGetOrdersForCustomer() {
		LOG.debug("testGetOrdersForCustomer");
		Customer customer = customerDao.getCustomer(1);
		assertEquals(new Integer(1), customer.getId());
		List<Order> list = orderDao.getOrdersForCustomer(customer);
		// for (Order order : list) {
		// // verarbeite order.getPizza()
		// }
	}

	public void testSelectByCustomerFilter() {
		OrderFilter filter = new OrderFilter();
		filter.setCustomerName("%ufe%");
		List<Order> list = orderDao.selectOrder(filter);
		assertEquals("list Groesse falsch?", 1, list.size());
		// pruefe Ergebnisse
		for (Order order : list) {
			assertTrue("Kunden-Filter fehlerhaft", (order.getCustomer()
					.getName().contains("ufe")));
		}
	}

	public void testSelectByCustomerExample() {
		Order exOrder = new Order();
		// creationTime & orderState werden beim Erzeugen gesetzt
		// hier zuruecksetzen falls nicht in der Abfrage erwuenscht
		exOrder.setCreationTime(null);
		exOrder.setState(null);
		Customer customer = new Customer();
		customer.setName("%ufe%");
		exOrder.setCustomer(customer);
		List<Order> list = orderDao.selectOrderByExample(exOrder);
		assertEquals("list Groesse falsch?", 1, list.size());
		// pruefe Ergebnisse
		for (Order order : list) {
			assertTrue("Kunden-Filter fehlerhaft", (order.getCustomer()
					.getName().contains("ufe")));
		}
	}

	public void testSelectByStateFilter() {
		OrderFilter filter = new OrderFilter();
		filter.setState(OrderState.OPEN);
		List<Order> list = orderDao.selectOrder(filter);
		assertTrue("leere liste?", list.size() > 0);
		// pruefe Ergebnisse
		for (Order order : list) {
			assertTrue("State-Filter fehlerhaft", order.getState().equals(
					OrderState.OPEN));
		}
	}

	public void testSelectByNameAndStateFilter() {
		OrderFilter filter = new OrderFilter();
		filter.setCustomerName("%ufe%");
		filter.setState(OrderState.OPEN);
		List<Order> list = orderDao.selectOrder(filter);
		assertTrue("leere liste?", list.size() > 0);
		// pruefe Ergebnisse
		for (Order order : list) {
			assertTrue("State-Filter fehlerhaft", order.getState().equals(
					OrderState.OPEN));
			assertTrue("Kunden-Filter fehlerhaft", (order.getCustomer()
					.getName().contains("ufe")));
		}
	}

	// REWRITE TO MATCH testSelectByNameAndStateFilter
	//
	// /** same as testSelectByNameFilter but uses an examplePizza */
	// public void testSelectByExampleName() {
	// Pizza exPizza = new Pizza();
	// exPizza.setName("Spicy%");
	// List<Pizza> list = pizzaDao.selectPizzaByExample(exPizza);
	// // Hat das Ergebnis die richtigen Namen?
	// for (Pizza pizza : list) {
	// assertTrue("Name-Filter fehlerhaft", (pizza.getName()
	// .contains("Spicy")));
	// }
	// }

	public void testSelectByPizzaFilter() {
		OrderFilter filter = new OrderFilter();
		filter.addPizzaId(2);

		List<Order> list = orderDao.selectOrder(filter);
		assertTrue("leere liste?", list.size() > 0);

		// Haben die Bestellungen die richtige Pizzas? (lazy loading)
		for (Order order : list) {
			assertTrue("Pizza-Filter fehlerhaft", (order.containsPizza(2)));
		}
	}

	// REWRITE AS testSelectByPizzaFilter
	// /** same as testSelectByToppingFilter but uses an examplePizza */
	// public void testSelectByExampleTopping() {
	// Pizza exPizza = new Pizza();
	// // unschoen
	// Topping top = new Topping();
	// top.setId(4);
	// exPizza.addTopping(top);
	// List<Pizza> list = pizzaDao.selectPizzaByExample(exPizza);
	// assertEquals("wrong nbr of pizzas", 2, list.size());
	// // Haben die Pizzas die richtige Belege`? (lazy loading)
	// for (Pizza pizza : list) {
	// assertTrue("Topping-Filter fehlerhaft", (pizza
	// .containsTopping(top)));
	// }
	// }
	//
	// /** same as testSelectByExampleTopping but with extra criteria */
	// public void testSelectByExampleTopping2() {
	// Pizza exPizza = new Pizza();
	// exPizza.setName("Tuna%");
	// // unschoen
	// Topping top = new Topping();
	// top.setId(4);
	// exPizza.addTopping(top);
	// List<Pizza> list = pizzaDao.selectPizzaByExample(exPizza);
	// assertEquals("wrong nbr of pizzas", 1, list.size());
	// // Haben die Pizzas die richtige Belege`? (lazy loading)
	// for (Pizza pizza : list) {
	// assertTrue("Topping-Filter fehlerhaft", (pizza
	// .containsTopping(top)));
	// }
	// }

	// TODO: tests für OrderFilter.creationDate

	public void setOrderDao(OrderDao anOrderDao) {
		this.orderDao = anOrderDao;
	}

	public void setCustomerDao(CustomerDao aCustomerDao) {
		this.customerDao = aCustomerDao;
	}

	@Override
	protected String[] getConfigLocations() {
		return new String[] { "applicationContext.xml" };
	}

}
