package de.hanser.buch.opiz.dao;

import org.apache.log4j.Logger;
import org.hibernate.SessionFactory;
import org.springframework.test.annotation.AbstractAnnotationAwareTransactionalTests;

import de.hanser.buch.opiz.domain.OrderState;
import de.hanser.buch.opiz.domain.Pizza;
import de.hanser.buch.opiz.domain.Topping;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;

/**
 * Tests fuer HibernatePizzaDaoImpl.
 * <p>
 * Junit 4 geht nicht, da die Spring-Testklassen TestCase aus JUnit 3 erweitern!
 */
public class HibernatePizzaDaoImplTest extends
		AbstractAnnotationAwareTransactionalTests {

	private static final Logger LOG = Logger
			.getLogger(HibernatePizzaDaoImplTest.class);

	/** Zugriff auf Pizza-DAO */
	private PizzaDao pizzaDao;

	public void setPizzaDao(PizzaDao aPizzaDao) {
		this.pizzaDao = aPizzaDao;
	}

	/** Zugriff auf hibernate SessionFactory - fuer flush-Aufruf */
	private SessionFactory sessionFactory;

	public void setSessionFactory(SessionFactory aSessionFactory) {
		this.sessionFactory = aSessionFactory;
	}

	@Override
	protected void onSetUpBeforeTransaction() {
	}

	@Override
	protected void onSetUpInTransaction() throws Exception {
		JDBCTestHelper.insertTopping(jdbcTemplate, 1, "Tomato sauce",
				ToppingPriceCategory.ONE);
		JDBCTestHelper.insertTopping(jdbcTemplate, 2, "Mozzarella",
				ToppingPriceCategory.TWO);
		JDBCTestHelper.insertTopping(jdbcTemplate, 3, "Salami",
				ToppingPriceCategory.TWO);
		JDBCTestHelper.insertTopping(jdbcTemplate, 4, "Onion",
				ToppingPriceCategory.ONE);
		JDBCTestHelper.insertTopping(jdbcTemplate, 5, "Tuna",
				ToppingPriceCategory.THREE);
		JDBCTestHelper.insertTopping(jdbcTemplate, 6, "Mushrooms",
				ToppingPriceCategory.THREE);
		JDBCTestHelper.insertOrder(jdbcTemplate, 1, OrderState.NEW, null);
		JDBCTestHelper.insertOrder(jdbcTemplate, 2, OrderState.OPEN, null);
		JDBCTestHelper.insertPizza(jdbcTemplate, 1, "Margherita",
				new int[] { 1 }, 1);
		JDBCTestHelper.insertPizza(jdbcTemplate, 2, "Spicy Cheese", new int[] {
				1, 2, 3 }, 1);
		JDBCTestHelper.insertPizza(jdbcTemplate, 3, "Tuna Fish", new int[] { 1,
				5, 4 }, 2);
		JDBCTestHelper.insertPizza(jdbcTemplate, 4, "Funghi", new int[] { 1, 4,
				6 }, 1);
	}

	public void testAddTopping() {
		final Integer PIZZA_ID = 1;
		Topping topping = new Topping(2);
		// pizzaDao.addTopping(topping, PIZZA_ID);
		Pizza aPizza = pizzaDao.getPizza(PIZZA_ID);
		aPizza.addTopping(topping);
		// flush anstossen, damit die Aenderung in der Datenbank sichtbar wird.
		// Alternativ waere, ein AOP-Interceptor zu benutzen.
		sessionFactory.getCurrentSession().flush();
		assertEquals(
				"falscher Anzahl Belaege: ",
				2,
				jdbcTemplate
						.queryForInt("select count(1) from pizza_topping where pizza_id = "
								+ PIZZA_ID));
	}

	public void testCreatePizza() {
		LOG.debug("start testCreatePizza");
		Pizza aPizza = new Pizza();
		aPizza.setName("testCreatePizza");
		assertNull("id nicht null?", aPizza.getId());
		Integer res = pizzaDao.savePizza(aPizza);
	}

	public void testUpdate() {
		final Integer PIZZA_ID = 1;
		Pizza aPizza = pizzaDao.getPizza(PIZZA_ID);
		System.out.println(aPizza);
		sessionFactory.getCurrentSession().flush();
		sessionFactory.getCurrentSession().clear();
		sessionFactory.getCurrentSession().refresh(aPizza);
		System.out.println("step1");
		aPizza.setName("Neuer Name");
		System.out.println("before update");
		pizzaDao.update(aPizza);
		System.out.println("after update");
		sessionFactory.getCurrentSession().flush();
		System.out.println("after flush");
	}

	@Override
	protected String[] getConfigLocations() {
		return new String[] { "applicationContext.xml" };
	}
}
