package de.hanser.buch.opiz.dao;

import java.util.List;
import java.util.Map;

import org.springframework.test.annotation.AbstractAnnotationAwareTransactionalTests;

import de.hanser.buch.opiz.domain.Topping;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;

/**
 * Tests fuer die HibernateToppingDaoImpl Klasse.
 */
public class HibernateToppingDaoImplTest extends
		AbstractAnnotationAwareTransactionalTests {

	private ToppingDao toppingDao;

	public void setToppingDao(ToppingDao aToppingDao) {
		this.toppingDao = aToppingDao;
	}

	@Override
	protected void onSetUpInTransaction() throws Exception {
		JDBCTestHelper.insertTopping(jdbcTemplate, 1, "Tomato sauce",
				ToppingPriceCategory.ONE);
		JDBCTestHelper.insertTopping(jdbcTemplate, 2, "Edam",
				ToppingPriceCategory.TWO);
		JDBCTestHelper.insertTopping(jdbcTemplate, 3, "Salami",
				ToppingPriceCategory.TWO);
	}

	public void testSelectByUnknownName() {
		List<Topping> list = toppingDao
				.getToppingByName("thisnamedoesnotexist");
		assertEquals("Liste sollte leer sein", 0, list.size());
	}

	public void testSelectByNameOneResult() {
		List<Topping> list = toppingDao.getToppingByName("Tomato");
		assertEquals("Liste sollte ein Eintrag enthalten", 1, list.size());
		assertEquals("Falscher Name", "Tomato sauce", list.get(0).getName());
	}

	public void testSelectByNameMultipleResults() {
		List<Topping> list = toppingDao.getToppingByName("am");
		assertEquals("Liste sollte zwei Eintraege enthalten", 2, list.size());
	}

	public void testSelectAll() {
		List<Topping> list = toppingDao.selectAll();
		assertEquals("Liste sollte zwei Eintraege enthalten", 3, list.size());
	}

	/** Testet das Aufrufen eines Named-Query */
	public void testSelectByNameAndPrice() {
		final String name = "%am%";
		List<Topping> list = toppingDao.selectByNameAndPrice(name,
				ToppingPriceCategory.TWO);
		assertEquals("falsche List-Größe?", 2, list.size());
		// pruefe Ergebnisse
		for (Topping topping : list) {
			assertTrue("falscher Name", topping.getName().contains("am"));
			assertTrue("falsche Preiskategorie", topping.getPriceCategory()
					.equals(ToppingPriceCategory.TWO));
		}
	}

	/** Testet das Paged-Aufrufen eines Named-Query */
	public void testSelectByNameAndPricePaged() {
		final String NAME = "%am%";
		final int PAGE_SIZE = 1;
		int startPage = 0;
		Topping topping1, topping2;
		List<Topping> list = toppingDao.selectByNameAndPricePaged(NAME,
				ToppingPriceCategory.TWO, startPage, PAGE_SIZE);
		assertEquals("falsche List-Größe?", 1, list.size());
		// pruefe Ergebnisse
		topping1 = list.get(0);
		assertTrue("falscher Name", topping1.getName().contains("am"));
		assertTrue("falsche Preiskategorie", topping1.getPriceCategory()
				.equals(ToppingPriceCategory.TWO));

		// zweiter Aufruf
		startPage = 1;
		list = toppingDao.selectByNameAndPricePaged(NAME,
				ToppingPriceCategory.TWO, startPage, PAGE_SIZE);
		assertEquals("falsche List-Größe?", 1, list.size());
		// pruefe Ergebnisse
		topping2 = list.get(0);
		assertTrue("falscher Name", topping2.getName().contains("am"));
		assertTrue("falsche Preiskategorie", topping2.getPriceCategory()
				.equals(ToppingPriceCategory.TWO));
		assertNotSame("objekte gleich?", topping1, topping2);
	}

	public void testGetNumberToppingsOrderedByPrice() {
		List result = toppingDao.getNumberToppingsOrderedByPrice();
		assertEquals("falsches Ergebnis", 2, result.size());
		// ein Topping in Preis-Kategorie 1
		System.out.println(((Map) result.get(0)));
		assertEquals("falsches category fuer 1. Eintrag",
				ToppingPriceCategory.ONE, ((Map) result.get(0)).get("category"));
		assertEquals("falsches count fuer 1. Eintrag", new Long(1),
				((Map) result.get(0)).get("anzahl"));
		// zwei Toppings in Preis-Kategorie 2
		assertEquals("falsches ToppingPriceCategory fuer 2. Eintrag",
				ToppingPriceCategory.TWO, ((Map) result.get(1)).get("category"));
		assertEquals("falsches count fuer 2. Eintrag", new Long(2),
				((Map) result.get(1)).get("anzahl"));
	}

	@Override
	protected String[] getConfigLocations() {
		return new String[] { "applicationContext.xml" };
	}

}
