package de.hanser.buch.opiz.dao;

import de.hanser.buch.opiz.domain.OrderState;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;
import org.springframework.jdbc.core.JdbcTemplate;

import java.sql.Types;
import java.util.Date;

/**
 * Fuegt per JDBC Zeilen in die Datenbank ein. <p/> In realem Testcode wuerden
 * wir auf JDBC verzichten und direkt in den Testklassen Hibernate einsetzen. Im
 * Buch hilft der JDBC-Code aber, einzelne Features von Hibernate isoliert
 * darzustellen.
 *
 * @author Stefan Wille
 */
public class JDBCTestHelper {
    /**
     * Fgt einen Belag per JDBC in die Datenbank ein.
     */
    public static void insertTopping(JdbcTemplate jdbcTemplate,
            int id, String name, ToppingPriceCategory priceCategory) {
        jdbcTemplate
                .update(
                        "insert into topping (id, name, priceCategory) values (?, ?, ?)",
                        new Object[] { id, name,
                                priceCategory.ordinal() });
    }

    /**
     * Fgt eine Pizza per JDBC in die Datenbank ein.
     */
    public static void insertPizza(JdbcTemplate jdbcTemplate, Integer id, String name,
                                   int[] toppings,
                                   Integer orderId) {
        jdbcTemplate.update("insert into pizza (id, name, order_fk) values(?, ?, ?)",
                new Object[]{id, name, orderId});

        for (int topping : toppings) {
            jdbcTemplate
                    .update(
                            "insert into pizza_topping (pizza_id, topping_id) values(?, ?)",
                            new Object[] { id, topping });
        }
    }

    /**
     * Fgt eine Bestellung per JDBC in die Datenbank ein.
     */
    public static void insertOrder(JdbcTemplate jdbcTemplate, int id,
            OrderState state, Integer customerId) {
        jdbcTemplate
                .update(
                        "insert into orders (id, state, creationTime, customer_fk) values(?, ?, ?, ?)",
                        new Object[] { id, state, new Date(),
                                customerId }, new int[] {
                                Types.INTEGER, Types.VARCHAR,
                                Types.TIMESTAMP, Types.INTEGER });
    }

    /**
     * Fgt einen Kunden per JDBC in die Datenbank ein.
     */
    public static void insertCustomer(JdbcTemplate jdbcTemplate,
            int id, String name, String user_name) {
        jdbcTemplate
                .update(
                        "insert into businesspartner (id, discriminator, name, user_name) values(?, ?, ?, ?)",
                        new Object[] { id, "Customer", name,
                                user_name });
    }

}
