package de.hanser.buch.opiz.domain;

import junit.framework.TestCase;

import java.util.ArrayList;
import java.util.List;

public class OrderTest extends TestCase {

    private Pizza pizza1;
    private Pizza pizza2;

    @Override
    public void setUp() throws Exception {
        super.setUp();
        Topping topping1 = new Topping("cheese", ToppingPriceCategory.TWO);
        Topping topping2 = new Topping("tomato", ToppingPriceCategory.THREE);

        List<Topping> toppings1 = new ArrayList<Topping>();
        toppings1.add(topping1);
        toppings1.add(topping2);
        pizza1 = new Pizza("my pizza 1", toppings1);

        List<Topping> toppings2 = new ArrayList<Topping>();
        toppings2.add(topping1);
        toppings2.add(topping2);
        pizza2 = new Pizza("my pizza 2", toppings2);
    }

    public void testAddPizzaInWrongStatus() {
        Order order = new Order();
        order.addPizza(pizza1);
        assertEquals(1, order.getPizzas().size());
        assertEquals("finishOrder fehlgeschlagen!", true, order.finishOrder());
        assertEquals("addPizza nicht fehlgeschlagen!", false, order.addPizza(pizza2));
    }

    public void testFinishOrderWrongStatus() {
        Order order = new Order();
        assertEquals("finishOrder fehlgeschlagen!", true, order.finishOrder());
        assertEquals("finishOrder nicht fehlgeschlagen!", false, order.finishOrder());
    }
    
    public void testAddPizza() {
        Order order = new Order();
        order.addPizza(pizza1);
        assertEquals(1, order.getPizzas().size());
        assertSame(pizza1, order.getPizzas().get(0));
        assertSame(order, pizza1.getOrder());
    }

    public void testRemovePizza() {
        Order order = new Order();
        order.addPizza(pizza1);
        order.removePizza(pizza1);
        assertEquals(0, order.getPizzas().size());
    }

    public void testPriceOfOnePizza() {
        Order order = new Order();
        order.addPizza(pizza1);

        // Der Preis ist der Preis von pizza1.
        assertEquals(order.getPrice(), pizza1.getPrice());
    }

    public void testPriceWithMultiplePizzas() {
        Order order = new Order();
        order.addPizza(pizza1);
        order.addPizza(pizza2);

        // Der Preis ist die Summe der Einzel-Pizza-Preise.
        assertEquals(order.getPrice(), pizza1.getPrice() + pizza2.getPrice());
    }

}
