package de.hanser.buch.opiz.service;

import org.springframework.test.annotation.AbstractAnnotationAwareTransactionalTests;

import de.hanser.buch.opiz.dao.OrderDao;
import de.hanser.buch.opiz.domain.Order;


/**
 * Testen der Klasse InventoryChecker erfolgt durch Durchfuehrung von Bestellungen.
 * Dabei muss ersichtlich sein, dass der Lagerbestand reduziert wird.
 * @author Gerald Bachlmayr
 *
 */
public class InventoryCheckerTest extends AbstractAnnotationAwareTransactionalTests {


    private Order order1;
    private Order order2;
    private OrderDao orderDao;
    private static int pizzaInitCount;

    
    @Override
    protected void onSetUpBeforeTransaction() throws Exception {
    	super.onSetUpBeforeTransaction();
    	InventoryChecker.setOrderCounter(0);
    	pizzaInitCount = InventoryChecker.getPizzaCounter();
    	
        order1 = TestUtil.createTestOrder1();
        
        order2 = TestUtil.createTestOrder2();
       
    }
    
	
    public void testAddPizza() throws InventoryException {
    	
    	// bisher sind null Bestellungen erfolgt
    	assertTrue(0 == InventoryChecker.getOrderCounter());
    	
    	// Pizzabestand darf sich noch nicht ver�ndert haben (Negativtest)
    	assertTrue(pizzaInitCount == InventoryChecker.getPizzaCounter());
    	
    	Integer id1 = orderDao.saveOrder(order1);
    	Integer id2 = orderDao.saveOrder(order2);
    	
        assertFalse(id1.equals(id2));
    	// zwei Bestellungen sind erfolgt
    	
    	assertTrue(2 == InventoryChecker.getOrderCounter());
    	
    	// Der Pizzabestand muss nun weniger sein
    	assertTrue(InventoryChecker.getPizzaCounter() < pizzaInitCount);
    }
    
	@Override
	protected String[] getConfigLocations() {
		return new String[]{"applicationContext.xml"};
	}


	public OrderDao getOrderDao() {
		return orderDao;
	}


	public void setOrderDao(OrderDao orderDao) {
		this.orderDao = orderDao;
	}

}