package de.hanser.buch.opiz.service;

import de.hanser.buch.opiz.dao.OrderDao;
import de.hanser.buch.opiz.dao.PizzaDao;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.Pizza;
import junit.framework.TestCase;
import org.easymock.EasyMock;

import java.util.ArrayList;
import java.util.List;

/**
 * Ein Beispiel, wie wir ein EasyMock-basierte JUnit-Testfall schreiben knnen.
 * <p>
 * NB: wir kommen ganz ohne Spring aus.
 * <p>
 * 
 * @author dr richard oates
 * 
 */
public class OrderServiceMockTest extends TestCase {

    private OrderServiceImpl orderService;

    private PizzaDao pizzaDaoMock;

    private OrderDao orderDaoMock;

    /**
     * Mock-Objekte initialisieren.
     */
    @Override
    protected void setUp() throws Exception {
        super.setUp();
        orderService = new OrderServiceImpl();

        // erzeuge Mock-Objekte
        pizzaDaoMock = EasyMock.createMock(PizzaDao.class);
        orderDaoMock = EasyMock.createMock(OrderDao.class);

        orderService.setOrderDao(orderDaoMock);
        orderService.setPizzaDao(pizzaDaoMock);
    }

    public void testAddPizzaToOrder() {
        final Integer ORDER_ID = 1;
        final Integer EXISTING_PIZZA_ID = 1;
        final Integer NEW_PIZZA_ID = 2;

        List<Pizza> pizzaList = new ArrayList<Pizza>();
        pizzaList.add(new Pizza(EXISTING_PIZZA_ID));
        Order order = new Order();
        order.setPizzas(pizzaList);

        EasyMock.expect(pizzaDaoMock.getPizza(NEW_PIZZA_ID))
                .andReturn(new Pizza(NEW_PIZZA_ID));
        EasyMock.expect(orderDaoMock.getOrder(ORDER_ID)).andReturn(
                order);
        // aendere Mocks Verhalten
        EasyMock.replay(pizzaDaoMock);
        EasyMock.replay(orderDaoMock);
        // rufe Servicemethode auf
        orderService.addPizzaToOrder(ORDER_ID, NEW_PIZZA_ID);
        // pruefe Ergebnis
        assertEquals("Pizza nicht erfolgreich hinzugefuegt: ", 2, order.getPizzas().size());
        EasyMock.verify(pizzaDaoMock);
        EasyMock.verify(orderDaoMock);
    }
}
