package de.hanser.buch.opiz.service;

import org.hibernate.SessionFactory;
import org.springframework.test.annotation.AbstractAnnotationAwareTransactionalTests;

import de.hanser.buch.opiz.dao.JDBCTestHelper;
import de.hanser.buch.opiz.domain.OrderState;
import de.hanser.buch.opiz.domain.ToppingPriceCategory;

/*
 * TODO: the order of these tests is important, but shouldn't be!
 */
public class OrderServiceTest extends AbstractAnnotationAwareTransactionalTests {
    private OrderService orderService;

    /** Zugriff auf Hibernate SessionFactory - fuer flush-Aufruf */
    private SessionFactory sessionFactory;

    @Override
    protected void onSetUpInTransaction() throws Exception {
        JDBCTestHelper.insertTopping(jdbcTemplate, 1, "Tomato sauce",
                ToppingPriceCategory.ONE);
        JDBCTestHelper.insertOrder(jdbcTemplate, 1, OrderState.NEW,
                null);
        JDBCTestHelper.insertPizza(jdbcTemplate, 1, "Margherita",
                new int[] { 1 }, 1);
    }

    public void testGetPrice() {
        double price = orderService.getPrice(1);
        assertEquals(3.95, price); // BASE_PRICE + Preis der Belag
    }

    public void testCreatePendingOrder() {
        final int PIZZA_ID = 2;
        JDBCTestHelper.insertPizza(jdbcTemplate, PIZZA_ID,
                "Margarita", new int[] { 1 }, null);
        Integer orderId = orderService.createPendingOrder(PIZZA_ID);
        sessionFactory.getCurrentSession().flush();
        assertTrue("Falsche orderId (==" + orderId + ")",
                orderId > 0);
        assertEquals(
                "Falsche orderId in pizza",
                orderId.intValue(),
                jdbcTemplate
                        .queryForInt("select order_fk from pizza where id = "
                                + PIZZA_ID));
    }

    public void testAddPizzaToOrder() {
        final int ORDER_ID = 1;
        final int PIZZA_ID = 2;
        JDBCTestHelper.insertPizza(jdbcTemplate, PIZZA_ID,
                "Margarita", new int[] { 1 }, null);
        assertTrue(orderService.addPizzaToOrder(ORDER_ID, PIZZA_ID));
        sessionFactory.getCurrentSession().flush();
        assertEquals(
                "Falsche Anzahl Pizzen fuer Order " + ORDER_ID,
                2,
                jdbcTemplate
                        .queryForInt("select count(1) from pizza where order_fk= "
                                + ORDER_ID));
    }

    public void testAddUnknownPizzaToOrder() {
        final int ORDER_ID = 1;
        final int PIZZA_ID = 2;
        try {
            assertTrue(orderService.addPizzaToOrder(ORDER_ID, PIZZA_ID));
            fail("exception erwartet");
        } catch (UnknownPizzaException x) {
            // ok
        }
    }

    public void testAddPizzaToUnknownOrder() {
        final int ORDER_ID = 2;
        final int PIZZA_ID = 2;
        JDBCTestHelper.insertPizza(jdbcTemplate, PIZZA_ID,
                "Margarita", new int[] { 1 }, null);
        try {
            orderService.addPizzaToOrder(ORDER_ID, PIZZA_ID);
            fail("exception erwartet");
        } catch (UnknownOrderException x) {
            // ok
        }
    }

    public void setOrderService(OrderService anOrderService) {
        this.orderService = anOrderService;
    }

    public void setSessionFactory(SessionFactory aSessionFactory) {
        this.sessionFactory = aSessionFactory;
    }

    @Override
    protected String[] getConfigLocations() {
        return new String[] { "applicationContext.xml" };
    }

}
