package de.hanser.buch.opiz.service;

import org.springframework.test.annotation.AbstractAnnotationAwareTransactionalTests;

import de.hanser.buch.opiz.dao.CustomerDao;
import de.hanser.buch.opiz.dao.OrderDao;
import de.hanser.buch.opiz.domain.Customer;
import de.hanser.buch.opiz.domain.Order;

/**
 * 
 * Testen der deklarativen Transaktion.
 * @author Gerald Bachlmayr
 *
 */
public class TxDeclarativeTest extends AbstractAnnotationAwareTransactionalTests {
	
	private OrderService orderService;
	private OrderDao orderDao;
	private CustomerDao customerDao;
	private org.springframework.orm.hibernate3.HibernateTransactionManager txMgr;
	private Order order1;
    private Integer custId;
	
	private boolean isTxActive = false;

	
	@Override
	protected void onSetUpBeforeTransaction() throws Exception {
		isTxActive = false;
        Customer customer = TestUtil.createCustomer();
        custId = customerDao.saveCustomer(customer);

        order1 = TestUtil.createTestOrder1();        
	}
	
	@Override
	protected void onSetUpInTransaction() throws Exception {
		isTxActive = true;
	}
	
	@Override
	protected void onTearDownAfterTransaction() throws Exception {
		setDirty();
		isTxActive = false;
	}
	
	/**
	 * Tested die annotierte deklarative Transaktion
	 * 
	 */
	public void testAnnotatedTx() throws Exception {
		int qTime = orderService.getQueueTime();
		assertTrue(isTxActive);
		assertTrue (0 < qTime);
	}
	
	/**
	 * Tested die XML-basierte deklarative Transaktion
	 *
	 */
	public void testCreateOrderWithCustomer() {
		assertNotNull(customerDao);
		// Testdaten
System.out.println(":: " + order1.getCustomer().getId() + "/" + custId);
        // Aufruf der zu testenden Methode
        orderService.createOrder(order1, custId);
    }

	
	@Override
	protected String[] getConfigLocations() {
		return new String[]{"applicationContext.xml"};
	}

	public OrderDao getOrderDao() {
		return orderDao;
	}

	public void setOrderDao(OrderDao orderDao) {
		this.orderDao = orderDao;
	}

	public OrderService getOrderService() {
		return orderService;
	}

	public void setTxoService(OrderService oService) {
		this.orderService = oService;
	}

	public org.springframework.orm.hibernate3.HibernateTransactionManager getTxMgr() {
		return txMgr;
	}

	public void setTxMgr(
			org.springframework.orm.hibernate3.HibernateTransactionManager txMgr) {
		this.txMgr = txMgr;
	}

	public CustomerDao getCustomerDao() {
		return customerDao;
	}

	public void setCustomerDao(CustomerDao customerDao) {
		this.customerDao = customerDao;
	}
	
	
}
