package de.hanser.buch.opiz.service;

import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.test.annotation.AbstractAnnotationAwareTransactionalTests;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import de.hanser.buch.opiz.dao.OrderDao;
import de.hanser.buch.opiz.domain.Order;
import de.hanser.buch.opiz.domain.OrderState;



/**
 * 
 * Testen von lokalen Transaktionen: Positiv- und Negativtests werden durchgefuehrt.
 * @author Gerald Bachlmayr
 *
 */
public class TxProgrammaticTest extends AbstractAnnotationAwareTransactionalTests {
		
	private Order order;
	private OrderDao orderDao;
	private org.springframework.orm.hibernate3.HibernateTransactionManager txMgr;
	private TransactionTemplate transactionTemplate;
	private HibernateTemplate hibernateTemplate;
	
	
	
	public void testHibernateTemplate() {
		getHibernateTemplate().save(order);
	}
	
    /**
     * Testen eines R�ckgabewertes
     */
	public void testTxTemplateWithRetVal() throws Exception {
		TransactionTemplate tt = new TransactionTemplate(txMgr);
		Object result = tt.execute(new TransactionCallback() {
			public Object doInTransaction(TransactionStatus status) {
				return orderDao.saveOrder(order);
			}
		});
		assertNotNull(result);
	}
	
    /**
     * Testen ohne R�ckgabewert
     */
	public void testTxTemplateWithoutRetVal() throws Exception {
		getTransactionTemplate().execute(new TransactionCallbackWithoutResult() {
			@Override
            public void doInTransactionWithoutResult(TransactionStatus status) {
				orderDao.saveOrder(order);
			}
		});
	}
	
	
	public void testTxPlatformTxManager() throws Exception {
        // Beginn der Transaktion festlegen
		DefaultTransactionDefinition def = new DefaultTransactionDefinition();
		def.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
		TransactionStatus status = txMgr.getTransaction(def);
		Integer result;
		try {
			result = orderDao.saveOrder(order);
		}
		catch (Exception e) {
			// Rollback
			txMgr.rollback(status);
			throw e;
		}
		// Commit
		txMgr.commit(status);
        assertNotNull(result);	
	}
	
	/**
	 * Testwerte werden geladen.
	 */
	@Override
	protected void onSetUpBeforeTransaction() throws Exception {
		super.onSetUpBeforeTransaction();
		
		// Testdaten
        order = TestUtil.createTestOrder1();
        order.setState(OrderState.OPEN);
	}
	
	@Override
	protected void onTearDownAfterTransaction() throws Exception {
		setDirty();
	}
	
	@Override
	protected String[] getConfigLocations() {
		return new String[]{"applicationContext.xml"};
	}


	public void setOrderDao(OrderDao orderDao) {
		this.orderDao = orderDao;
	}

	public org.springframework.orm.hibernate3.HibernateTransactionManager getTxMgr() {
		return txMgr;
	}

	public void setTxMgr(
			org.springframework.orm.hibernate3.HibernateTransactionManager txMgr) {
		this.txMgr = txMgr;
	}

	public HibernateTemplate getHibernateTemplate() {
		return hibernateTemplate;
	}

	public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
		this.hibernateTemplate = hibernateTemplate;
	}

	public TransactionTemplate getTransactionTemplate() {
		return transactionTemplate;
	}

	public void setTransactionTemplate(TransactionTemplate transactionTemplate) {
		this.transactionTemplate = transactionTemplate;
	}

}
